//+------------------------------------------------------------------+
//|                                    Traders Dynamic Index.mq4     |
//|                                    Copyright  2006, Dean Malone |
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//|                                                                  |
//|                     Traders Dynamic Index                        |
//|                                                                  |
//|  This hybrid indicator is developed to assist traders in their   |
//|  ability to decipher and monitor market conditions related to    |
//|  trend direction, market strength, and market volatility.        |
//|                                                                  | 
//|  Even though comprehensive, the T.D.I. is easy to read and use.  |
//|                                                                  |
//|  Green line = RSI Price line                                     |
//|  Red line = Trade Signal line                                    |
//|  Blue lines = Volatility Band                                    | 
//|  Yellow line = Market Base Line                                  |  
//|                                                                  |
//|  Trend Direction - Immediate and Overall                         |
//|   Immediate = Green over Red...price action is moving up.        |
//|               Red over Green...price action is moving down.      |
//|                                                                  |   
//|   Overall = Yellow line trends up and down generally between the |
//|             lines 32 & 68. Watch for Yellow line to bounces off  |
//|             these lines for market reversal. Trade long when     |
//|             price is above the Yellow line, and trade short when |
//|             price is below.                                      |        
//|                                                                  |
//|  Market Strength & Volatility - Immediate and Overall            |
//|   Immediate = Green Line - Strong = Steep slope up or down.      | 
//|                            Weak = Moderate to Flat slope.        |
//|                                                                  |               
//|   Overall = Blue Lines - When expanding, market is strong and    |
//|             trending. When constricting, market is weak and      |
//|             in a range. When the Blue lines are extremely tight  |                                                       
//|             in a narrow range, expect an economic announcement   | 
//|             or other market condition to spike the market.       |
//|                                                                  |               
//|                                                                  |
//|  Entry conditions                                                |
//|   Scalping  - Long = Green over Red, Short = Red over Green      |
//|   Active - Long = Green over Red & Yellow lines                  |
//|            Short = Red over Green & Yellow lines                 |    
//|   Moderate - Long = Green over Red, Yellow, & 50 lines           |
//|              Short= Red over Green, Green below Yellow & 50 line |
//|                                                                  |
//|  Exit conditions*                                                |   
//|   Long = Green crosses below Red                                 |
//|   Short = Green crosses above Red                                |
//|   * If Green crosses either Blue lines, consider exiting when    |
//|     when the Green line crosses back over the Blue line.         |
//|                                                                  |
//|                                                                  |
//|  IMPORTANT: The default settings are well tested and proven.     |
//|             But, you can change the settings to fit your         |
//|             trading style.                                       |
//|                                                                  |
//|                                                                  |
//|  Price & Line Type settings:                           |                
//|   RSI Price settings                                             |               
//|   0 = Close price     [DEFAULT]                                  |               
//|   1 = Open price.                                                |               
//|   2 = High price.                                                |               
//|   3 = Low price.                                                 |               
//|   4 = Median price, (high+low)/2.                                |               
//|   5 = Typical price, (high+low+close)/3.                         |               
//|   6 = Weighted close price, (high+low+close+close)/4.            |               
//|                                                                  |               
//|   RSI Price Line & Signal Line Type settings                                   |               
//|   0 = Simple moving average       [DEFAULT]                      |               
//|   1 = Exponential moving average                                 |               
//|   2 = Smoothed moving average                                    |               
//|   3 = Linear weighted moving average                             |               
//|                                                                  |
//|   Good trading,                                                  |   
//|                                                                  |
//|   Dean                                                           |                              
//+------------------------------------------------------------------+

#property indicator_separate_window

#property indicator_buffers 10
#property indicator_color1 Black
#property indicator_color2 Black
#property indicator_color3 Black
#property indicator_color4 Black
#property indicator_color5 Green
#property indicator_color6 Red
#property indicator_color7 Blue
#property indicator_color8 Red
#property indicator_color9 Blue
#property indicator_color10 Red

extern int  MaxDetectDivergence = 10;
extern int  MinDetectDivergence = 1;
extern int  RSI_Period = 13;         //8-25
extern int  RSI_Price = 0;           //0-6
extern int  Volatility_Band = 34;    //20-40

extern int  RSI_Price_Line = 2;      
extern int  RSI_Price_Type = 0;      //0-3
extern int  Trade_Signal_Line = 7;   
extern int  Trade_Signal_Type = 0;   //0-3

extern bool       showClassicalDivergence=true;
extern bool       showHiddenDivergence=true;
extern color      bullishLineColor = clrBlue;
extern color      bearishLineColor = clrRed;

extern bool       displayAlert=false;


double RSIBuf[],UpZone[],MdZone[],DnZone[],MaBuf[],MbBuf[];

double bullishClassicalDivergence[];
double bearishClassicalDivergence[];
double bullishHiddenDivergence[];
double bearishHiddenDivergence[];

datetime lastAlertTime;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+


int init()
{
   IndicatorShortName("TDI");
   SetIndexBuffer(0,RSIBuf);
   SetIndexBuffer(1,UpZone);
   SetIndexBuffer(2,MdZone);
   SetIndexBuffer(3,DnZone);
   SetIndexBuffer(4,MaBuf);
   SetIndexBuffer(5,MbBuf);
   
   SetIndexStyle(0,DRAW_NONE); 
   SetIndexStyle(1,DRAW_LINE); 
   SetIndexStyle(2,DRAW_LINE,0,2);
   SetIndexStyle(3,DRAW_LINE);
   SetIndexStyle(4,DRAW_LINE,0,2);
   SetIndexStyle(5,DRAW_LINE,0,2);
   
   SetIndexLabel(0,NULL); 
   SetIndexLabel(1,"VB High"); 
   SetIndexLabel(2,"Market Base Line"); 
   SetIndexLabel(3,"VB Low"); 
   SetIndexLabel(4,"RSI Price Line");
   SetIndexLabel(5,"Trade Signal Line");
 
   SetLevelValue(0,50);
   SetLevelValue(1,68);
   SetLevelValue(2,32);
   SetLevelStyle(STYLE_DOT,1,DimGray);
   
   SetIndexBuffer(6,bullishClassicalDivergence);
   SetIndexBuffer(7,bearishClassicalDivergence);
   SetIndexStyle(6,DRAW_ARROW);
   SetIndexStyle(7,DRAW_ARROW);
   SetIndexArrow(6,233);
   SetIndexArrow(7,234);
   SetIndexLabel(6,"Classiscal Bullish Signal");
   SetIndexLabel(7,"Classiscal Bearish Signal");
   
   SetIndexBuffer(8,bullishHiddenDivergence);
   SetIndexBuffer(9,bearishHiddenDivergence);
   SetIndexStyle(8,DRAW_ARROW);
   SetIndexStyle(9,DRAW_ARROW);
   SetIndexArrow(8,233);
   SetIndexArrow(9,234);
   SetIndexLabel(8,"Hidden Bullish Signal");
   SetIndexLabel(9,"Hidden Bearish Signal");
   
   
   return(0);
}

//---

int deinit()
{
   ObjectsDeleteAll(0, "DivergenceLine");
   return(0);
}

//+------------------------------------------------------------------+

int start()
{

   double MA,RSI[];
   ArrayResize(RSI,Volatility_Band);
   int counted_bars=IndicatorCounted();
   int limit = Bars-counted_bars-1;
   int i;
   
   
   for(i=limit; i>=0; i--)
   {
      RSIBuf[i] = (iRSI(NULL,0,RSI_Period,RSI_Price,i)); 
      MA = 0;
      for(int x=i; x<i+Volatility_Band; x++) {
         RSI[x-i] = RSIBuf[x];
         MA += RSIBuf[x]/Volatility_Band;
      }
      UpZone[i] = (MA + (1.6185 * StDev(RSI,Volatility_Band)));
      DnZone[i] = (MA - (1.6185 * StDev(RSI,Volatility_Band)));  
      MdZone[i] = ((UpZone[i] + DnZone[i])/2);
   }
   for (i=limit;i>=0;i--)  
   {
      MaBuf[i] = (iMAOnArray(RSIBuf,0,RSI_Price_Line,0,RSI_Price_Type,i));
      MbBuf[i] = (iMAOnArray(RSIBuf,0,Trade_Signal_Line,0,Trade_Signal_Type,i));   
   } 
   //----
   
   
   for(i=limit + MaxDetectDivergence; i>=0; i--)
   {
      CatchBullishDivergence(i + 1);
      CatchBearishDivergence(i + 1);
   }
   
   
   
   //////
   return(0);
}


//+------------------------------------------------------------------+

double StDev(double& Data[], int Per)
{
   return(MathSqrt(Variance(Data,Per)));
}

//---

double Variance(double& Data[], int Per)
{
   double sum, ssum;
   for (int i=0; i<Per; i++)
   {
      sum += Data[i];
      ssum += MathPow(Data[i],2);
   }
   return((ssum*Per - sum*sum)/(Per*(Per-1)));
}
//+------------------------------------------------------------------+

void CatchBullishDivergence(int shift)
{
   if(IsIndicatorTrough(shift)==false) return;
   
   int currentTrough=shift;
   int lastTrough=GetIndicatorLastTrough(shift);
   
   if (
         (lastTrough>=0 && lastTrough<Bars) &&
         (MinDetectDivergence <= 0 || lastTrough - shift >= MinDetectDivergence) &&
         (MaxDetectDivergence <= 0 || lastTrough - shift <= MaxDetectDivergence) &&
         MaBuf[currentTrough]>MaBuf[lastTrough] &&
         Low[currentTrough]<Low[lastTrough] &&
         MaBuf[currentTrough] < MbBuf[currentTrough] && MaBuf[lastTrough] < MbBuf[lastTrough]
      )
   {
      bullishClassicalDivergence[currentTrough]=MaBuf[currentTrough];
      if(showClassicalDivergence==true)
      {
         DrawPriceTrendLine(Time[currentTrough],Time[lastTrough],Low[currentTrough], Low[lastTrough],bullishLineColor,STYLE_SOLID);
         DrawIndicatorTrendLine(Time[currentTrough],Time[lastTrough],MaBuf[currentTrough], MaBuf[lastTrough],bullishLineColor,STYLE_SOLID);
      }
      if(displayAlert==true)
         DisplayAlert("Classical bullish divergence on: ",currentTrough);
   }
   
   
   if (
         (lastTrough>=0 && lastTrough<Bars) &&
         (MinDetectDivergence <= 0 || lastTrough - shift >= MinDetectDivergence) &&
         (MaxDetectDivergence <= 0 || lastTrough - shift <= MaxDetectDivergence) &&
         MaBuf[currentTrough]<MaBuf[lastTrough] &&
         Low[currentTrough]>Low[lastTrough]
      )
   {
      bullishHiddenDivergence[currentTrough]=MaBuf[currentTrough];
      if(showHiddenDivergence==true)
      {
         DrawPriceTrendLine(Time[currentTrough],Time[lastTrough],Low[currentTrough], Low[lastTrough],bullishLineColor,STYLE_DOT);
         DrawIndicatorTrendLine(Time[currentTrough],Time[lastTrough],MaBuf[currentTrough], MaBuf[lastTrough],bullishLineColor,STYLE_DOT);
      }
      if(displayAlert==true)
         DisplayAlert("Hidden bullish divergence on: ",currentTrough);
   }
}
//+------------------------------------------------------------------+

void CatchBearishDivergence(int shift)
{
   if(IsIndicatorPeak(shift)==false) return;
   int currentPeak=shift;
   int lastPeak=GetIndicatorLastPeak(shift);
   
   if (
         (lastPeak>=0 && lastPeak<Bars) &&
         (MinDetectDivergence <= 0 || lastPeak - shift >= MinDetectDivergence) &&
         (MaxDetectDivergence <= 0 || lastPeak - shift <= MaxDetectDivergence) &&
         MaBuf[currentPeak]<MaBuf[lastPeak] && 
         High[currentPeak]>High[lastPeak] &&
         MaBuf[currentPeak] > MbBuf[currentPeak] && MaBuf[lastPeak] > MbBuf[lastPeak]
      )
   {
      bearishClassicalDivergence[currentPeak]=MaBuf[currentPeak];
      if(showClassicalDivergence==true)
      {
         DrawPriceTrendLine(Time[currentPeak],Time[lastPeak],High[currentPeak], High[lastPeak],bearishLineColor,STYLE_SOLID);
         DrawIndicatorTrendLine(Time[currentPeak],Time[lastPeak],MaBuf[currentPeak], MaBuf[lastPeak],bearishLineColor,STYLE_SOLID);
      }
      if(displayAlert==true)
         DisplayAlert("Classical bearish divergence on: ",currentPeak);
   }
   
   if(
         (lastPeak>=0 && lastPeak<Bars) && 
         (MinDetectDivergence <= 0 || lastPeak - shift >= MinDetectDivergence) &&
         (MaxDetectDivergence <= 0 || lastPeak - shift <= MaxDetectDivergence) &&
         MaBuf[currentPeak]>MaBuf[lastPeak] &&
         High[currentPeak]<High[lastPeak]
      )
   {
      bearishHiddenDivergence[currentPeak]=MaBuf[currentPeak];
      if(showHiddenDivergence==true)
      {
         DrawPriceTrendLine(Time[currentPeak],Time[lastPeak],High[currentPeak], High[lastPeak],bearishLineColor,STYLE_DOT);
         DrawIndicatorTrendLine(Time[currentPeak],Time[lastPeak],MaBuf[currentPeak], MaBuf[lastPeak],bearishLineColor,STYLE_DOT);
      }
      if(displayAlert==true)
         DisplayAlert("Hidden bearish divergence on: ",currentPeak);
   }
}

//+------------------------------------------------------------------+

bool IsIndicatorPeak(int shift)
{
   if(shift < 3) return(false);
   if(
         MaBuf[shift] > MaBuf[shift+1] && MaBuf[shift] > MaBuf[shift+2] &&
         MaBuf[shift] > MaBuf[shift-1] && MaBuf[shift] > MaBuf[shift-2]
      )
   {
      return(true);
   }
   return(false);
}

//+------------------------------------------------------------------+

bool IsIndicatorTrough(int shift)
{
   if(shift < 3) return(false);
   if(
         MaBuf[shift]< MaBuf[shift+1] && MaBuf[shift]< MaBuf[shift+2] &&
         MaBuf[shift] < MaBuf[shift-1] && MaBuf[shift] < MaBuf[shift-2]
      )
   {
      return(true);
   }
   return(false);
}

//+------------------------------------------------------------------+

int GetIndicatorLastPeak(int shift)
{
   for(int i = shift + 5; (i < Bars-3) && (i > 1); i++)
   {
      if(MaBuf[i] >= MaBuf[i+1] && MaBuf[i] > MaBuf[i+2] &&
         MaBuf[i] >= MaBuf[i-1] && MaBuf[i] > MaBuf[i-2])
      return(i);
   }
   return(-1);
}

//+------------------------------------------------------------------+

int GetIndicatorLastTrough(int shift)
{
   for(int i=shift + 5;(i<Bars-3) && (i>1); i++)
   {
      if(MaBuf[i] <= MaBuf[i+1] && MaBuf[i] < MaBuf[i+2] &&
         MaBuf[i] <= MaBuf[i-1] && MaBuf[i] < MaBuf[i-2])
      return(i);
   }
   return(-1);
}

//+------------------------------------------------------------------+

void DisplayAlert(string message,int shift)
{
   if(shift<=2 && Time[shift]!=lastAlertTime)
   {
      lastAlertTime=Time[shift];
      Alert(message, Symbol()," , chart ", TFName(Period()));
   }
}

//+------------------------------------------------------------------+

void DrawPriceTrendLine(datetime x1,datetime x2,double y1,
                        double y2,color lineColor,double style)
{
   string label="DivergenceLine# "+DoubleToStr(x1,0);
   ObjectDelete(label);
   ObjectCreate(label,OBJ_TREND,0,x1,y1,x2,y2,0,0);
   ObjectSet(label,OBJPROP_RAY,0);
   ObjectSet(label,OBJPROP_COLOR,lineColor);
   ObjectSet(label,OBJPROP_STYLE,style);
}

//+------------------------------------------------------------------+

void DrawIndicatorTrendLine(datetime x1,datetime x2,double y1,
                            double y2,color lineColor,double style)
{
   int indicatorWindow=WindowFind("TDI");
   if(indicatorWindow<0) return;
   string label="DivergenceLine$# "+DoubleToStr(x1,0);
   ObjectDelete(label);
   ObjectCreate(label,OBJ_TREND,indicatorWindow,x1,y1,x2,y2,0,0);
   ObjectSet(label,OBJPROP_RAY,0);
   ObjectSet(label,OBJPROP_COLOR,lineColor);
   ObjectSet(label,OBJPROP_STYLE,style);
}
//+------------------------------------------------------------------+

string TFName (int p)
{
   switch (p)
   {
      case PERIOD_M1: return ("M1");
      case PERIOD_M5: return ("M5");      
      case PERIOD_M15: return ("M15");
      case PERIOD_M30: return ("M30");
      case PERIOD_H1: return ("H1");
      case PERIOD_H4: return ("H4");
      case PERIOD_D1: return ("D1");
      case PERIOD_W1: return ("W1");
      case PERIOD_MN1: return ("MN1");
   }
   return("M" + p);
}

//+------------------------------------------------------------------+
